@@ -2,7 +2,7 @@ from __future__ import unicode_literals
 
 import logging
 
-from dvc.exceptions import CheckoutErrorSuggestGit
+from dvc.exceptions import CheckoutErrorSuggestGit, CheckoutError
 from dvc.progress import Tqdm
 
 
@@ -23,7 +23,9 @@ def get_all_files_numbers(stages):
     return sum(stage.get_all_files_number() for stage in stages)
 
 
-def checkout(self, target=None, with_deps=False, force=False, recursive=False):
+def _checkout(
+    self, target=None, with_deps=False, force=False, recursive=False
+):
     from dvc.stage import StageFileDoesNotExistError, StageFileBadNameError
 
     all_stages = self.stages()
@@ -40,6 +42,7 @@ def checkout(self, target=None, with_deps=False, force=False, recursive=False):
         total = get_all_files_numbers(stages)
         if total == 0:
             logger.info("Nothing to do")
+        failed = []
         with Tqdm(
             total=total, unit="file", desc="Checkout", disable=total == 0
         ) as pbar:
@@ -52,4 +55,10 @@ def checkout(self, target=None, with_deps=False, force=False, recursive=False):
                         )
                     )
 
-                stage.checkout(force=force, progress_callback=pbar.update_desc)
+                failed.extend(
+                    stage.checkout(
+                        force=force, progress_callback=pbar.update_desc
+                    )
+                )
+        if failed:
+            raise CheckoutError(failed)
