@@ -1,8 +1,10 @@
 from __future__ import unicode_literals
 
-from dvc.repo.scm_context import scm_context
+from . import locked
+from .scm_context import scm_context
 
 
+@locked
 @scm_context
 def run(self, no_exec=False, **kwargs):
     from dvc.stage import Stage
