@@ -1,3 +1,7 @@
+from . import locked
+
+
+@locked
 def update(self, target):
     from dvc.stage import Stage
 
